#define STRICT
#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* USAGE: huffc1mk < huffcode > huffcode.out */

FILE* in;
FILE* out;

char line[1024];

typedef struct _NODE {
	short next[2];
} NODE;
NODE nodes[8192];
int node_cnt;

void readline();
void run(int i_table);

int
main(int argc, char* argv[])
{
	in = stdin;
	if(argc >= 1 + 1) {
		in = fopen(argv[1], "r");
		if(!in) abort();
	}

	out = stdout;
	if(argc >= 1 + 2) {
		out = fopen(argv[2], "w");
		if(!out) abort();
	}

	run(32);
	run(33);

	if(in != stdin) fclose(in);
	if(out != stdout) fclose(out);

	fprintf(stderr, "OK\n");
	Sleep(500);
	return 0;
}

void
readline()
{
	int i;

	do {
		if(!fgets(line, sizeof line, in)) abort();
	} while(line[0] == '#' || line[0] == '\n');

	i = strlen(line) - 1;
	if(line[i] != '\n') abort();
	line[i] = '\0';
}

void
run(int i_table)
{
	int retval, i, a, b, c, vwxy, hlen;
	char hcod[100];
	int i_node, i_hlen, bit;

	for(;;) {
		readline();
		retval = sscanf(line, ".table%d%d%d%d", &i, &a, &b, &c);
		if(retval == 4 && i == i_table) break;
	}

	memset(nodes, 0, sizeof nodes);
	node_cnt = 1;

	for(i = 0; i < 16; i++) {
		readline();
		retval = sscanf(line, "%d%d%s", &vwxy, &hlen, hcod);
		if(retval != 3) abort();
		if(vwxy != i) abort();
		if(hlen != (int)strlen(hcod)) abort();

		i_node = 0;
		for(i_hlen = 0; i_hlen < hlen - 1; i_hlen++) {
			bit = hcod[i_hlen] - '0';
			if(!(bit == 0 || bit == 1)) abort();
			if(!nodes[i_node].next[bit]) {
				nodes[i_node].next[bit] = node_cnt++;
			}
			i_node = nodes[i_node].next[bit];
		}
		bit = hcod[i_hlen] - '0';
		if(!(bit == 0 || bit == 1)) abort();
		nodes[i_node].next[bit] = -vwxy;
	}

	fprintf(out, "static const short huffman_count1_table%d[][2] = {\n", i_table - 32);
	for(i_node = 0; i_node < node_cnt; i_node++) {
		fprintf(out, "\t/*%3d*/\t{%4d,%4d},\n", i_node, nodes[i_node].next[0], nodes[i_node].next[1]);
	}
	fprintf(out, "};\n");
}
